""" Docker Class that integrates with local docker environment """
import logging
import docker


class Api:
    """ API interface """

    def __init__(self):
        self.docker_client = docker.from_env()

    def download(self, image=None, tag="latest"):
        """ Download images from docker.io """
        if image is None:
            raise Exception(f"Error (download): Docker image name missing ({image})")
        logging.info(f"Docker downloading {image}:{tag}")
        download = self.docker_client.images.pull(image, tag)
        data = {"image": image, "tag": tag}

        return data

    def save(self, image=None, tag="latest"):
        """ Save images as tar files """
        if image is None:
            raise Exception(f"Error (save): Docker image name missing ({image})")
        logging.info(f"Docker saving {image}:{tag}")
        tar = self.docker_client.images.get(f"{image}:{tag}")

        filename = f"{image.replace('/', '_')}-{tag}.tar"
        f = open(filename, "wb")
        for chunk in tar.save(named=True):
            f.write(chunk)
        f.close()

        data = {"image": image, "tag": tag, "filename": filename}

        return data



import os
import base64
import json
import logging
import requests
import urllib3
from requests.auth import HTTPBasicAuth
from requests_toolbelt.multipart.encoder import MultipartEncoder
import xmltodict
from ciscodnacapphosting import dockerctl
from ciscodnacapphosting import cli


"""ciscodnacapphosting  Console Script.
Copyright (c) 2020 Cisco and/or its affiliates.
This software is licensed to you under the terms of the Cisco Sample
Code License, Version 1.1 (the "License"). You may obtain a copy of the
License at
               https://developer.cisco.com/docs/licenses
All use of the material herein must be in accordance with the terms of
the License. All rights not expressly granted by the License are
reserved. Unless required by applicable law or agreed to separately in
writing, software distributed under the License is distributed on an "AS
IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
or implied.
"""

author = "Robert Csapo"
email = "rcsapo@cisco.com"
description = "Cisco DNA Center App Hosting SDK"
repo_url = "https://github.com/robertcsapo/cisco-dnac-app-hosting-import-docker-images/"
copyright = "Copyright (c) 2020 Cisco and/or its affiliates."
license = "Cisco Sample Code License, Version 1.1"
version = "0.1"

urllib3.disable_warnings()
logging.getLogger(__name__).addHandler(logging.NullHandler())
logger = logging.getLogger(__name__)


class Api:
    def __init__(self):
        self.docker = dockerctl.Api()
        self.settings = {}

        """
        Read config from OS or local file
        """
        if "DNAC_CONFIG" in os.environ:
            config = self.config(config=os.environ["DNAC_CONFIG"], operation="decode")
        else:
            config = self.config(operation="read")

        if config[0] is False:
            logging.fatal(f"{config[1]}")
            raise Exception(f"Error: {config[1]}")

        self.settings = {**self.settings, **config[1]}
        self._auth()
        return

    def config(hostname="", username="", password="", secure="", **kwargs):
        if "encode" in kwargs["operation"]:
            data = {
                "dnac": {
                    "hostname": hostname,
                    "username": username,
                    "password": password,
                    "secure": secure,
                }
            }
            data = str(json.dumps(data, indent=4))
            data_base64 = base64.b64encode(data.encode("ascii"))
            try:
                with open("config.json", "w") as f:
                    f.write(data_base64.decode("utf-8"))
                f.close()
                return True, data_base64.decode("utf-8")
            except Exception as e:
                return False, f"Can't update config file ({e})"
            pass

        if "decode" in kwargs["operation"]:
            data = kwargs["config"]
            try:
                data == base64.b64encode(base64.b64decode(data)).decode("utf-8")
                data = json.loads(base64.b64decode(data).decode("utf-8"))
            except Exception as e:
                return False, f"Can't decode config ({e})"
            return True, data

        if "write" in kwargs["operation"]:
            data = {
                "dnac": {
                    "hostname": hostname,
                    "username": username,
                    "password": password,
                    "secure": secure,
                }
            }
            try:
                with open("config.json", "w") as f:
                    f.write(json.dumps(data, indent=4))
                f.close()
                return True, None
            except Exception as e:
                return False, f"Can't update config file ({e})"

        if "read" in kwargs["operation"]:
            try:
                with open("config.json", "r") as f:
                    data = f.read()
                f.close()
                try:
                    data == base64.b64encode(base64.b64decode(data)).decode("utf-8")
                    data = json.loads(base64.b64decode(data).decode("utf-8"))
                except Exception:
                    try:
                        data = json.loads(data)
                    except Exception:
                        return False, f"Can't load json from config"
                return True, data
            except Exception as e:
                return False, f"Can't read config file ({e})"

        return False, "Error"

    def _auth(self):
        """ Authenticate towards Cisco DNA Center """
        url = (
            f"https://{self.settings['dnac']['hostname']}/dna/system/api/v1/auth/token"
        )
        logging.info(f"Cisco DNA Center Authentication ({url})")
        data = self._request(type="auth", url=url)
        self.settings["dnac"]["token"] = data["Token"]
        return

    def get(self, **kwargs):
        """ Get Application(s) from Cisco DNA Center """
        if "image" in kwargs:
            logging.info(f"Cisco DNA Center AppHosting App ({kwargs['image']})")
            url = f"https://{self.settings['dnac']['hostname']}/api/iox/service/api/v1/appmgr/apps?searchByName={kwargs['image']}"
            if "tag" in kwargs:
                url = f"https://{self.settings['dnac']['hostname']}/api/iox/service/api/v1/appmgr/apps/{kwargs['image']}/{kwargs['tag']}"
        elif "appId" in kwargs:
            logging.info(f"Cisco DNA Center AppHosting App ({kwargs['appId']})")
            url = f"https://{self.settings['dnac']['hostname']}/api/iox/service/api/v1/appmgr/apps/{kwargs['appId']}/latest"
            if "tag" in kwargs:
                url = f"https://{self.settings['dnac']['hostname']}/api/iox/service/api/v1/appmgr/apps/{kwargs['appId']}/{kwargs['tag']}"
        else:
            logging.info(f"Cisco DNA Center AppHosting App List")
            url = f"https://{self.settings['dnac']['hostname']}/api/iox/service/api/v1/appmgr/apps?limit=1000&offset=0"

        data = self._request(type="get", url=url)
        return data

    def upload(self, **kwargs):
        """ Upload Applications to Cisco DNA Center """
        url = f"https://{self.settings['dnac']['hostname']}/api/iox/service/api/v1/appmgr/apps?type=docker"
        logging.info(f"Cisco DNA Center AppHosting Upload ({kwargs['tar']})")
        data = self._request(type="post", url=url, tar=kwargs["tar"])
        """ Categories is mandatory in the UI """
        if "categories" in kwargs:
            data = self.update(
                appId=data["appId"],
                tag=data["version"],
                categories=kwargs["categories"],
            )
            pass
        else:
            kwargs["categories"] = "Others"
            data = self.update(
                appId=data["appId"],
                tag=data["version"],
                categories=kwargs["categories"],
            )

        return data

    def upgrade(self, **kwargs):
        """ Upgrade Applications to Cisco DNA Center """
        if "tag" not in kwargs:
            app = self.get(appId=kwargs["appId"])
            kwargs["tag"] = app["version"]
        url = f"https://{self.settings['dnac']['hostname']}/api/iox/service/api/v1/appmgr/apps/{kwargs['appId']}/{kwargs['tag']}/package?type=docker&action=update"
        logging.info(f"Cisco DNA Center AppHosting Upgrade ({kwargs['tar']})")
        data = self._request(type="post", url=url, tar=kwargs["tar"])

        """ Categories is mandatory in the UI """
        if "categories" in kwargs:
            data = self.update(
                appId=data["appId"],
                tag=data["version"],
                categories=kwargs["categories"],
            )
            pass
        else:
            kwargs["categories"] = "Others"
            data = self.update(
                appId=data["appId"],
                tag=data["version"],
                categories=kwargs["categories"],
            )

        return data

    def update(self, **kwargs):
        """ Update Metadata about Applications to Cisco DNA Center """
        app = self.get(appId=kwargs["appId"])
        if "tag" in kwargs:
            url = f"https://{self.settings['dnac']['hostname']}/api/iox/service/api/v1/appmgr/apps/{kwargs['appId']}/{kwargs['tag']}"
        else:
            url = f"https://{self.settings['dnac']['hostname']}/api/iox/service/api/v1/appmgr/apps/{kwargs['appId']}/latest"

        """ Validate categories input, based on those in UI """
        valid_metadata = self._supported_app_metadata(**kwargs)
        if valid_metadata[0] is False:
            raise Exception(f"Error: Unsupported metadata for application {kwargs}")

        data = {**app, **valid_metadata[1]}
        logging.info(f"Cisco DNA Center AppHosting Update App ({data['name']})")
        data = self._request(type="put", url=url, payload=data)

        return data

    def delete(self, **kwargs):
        """ Delete Application(s) from Cisco DNA Center """
        if "tag" in kwargs:
            url = f"https://{self.settings['dnac']['hostname']}/api/iox/service/api/v1/appmgr/apps/{kwargs['appId']}/{kwargs['tag']}?cancelOutstandingActions=true"
        else:
            url = f"https://{self.settings['dnac']['hostname']}/api/iox/service/api/v1/appmgr/apps/{kwargs['appId']}/latest?cancelOutstandingActions=true"
        logging.info(f"Cisco DNA Center AppHosting Delete App ({kwargs['appId']})")
        data = self._request(type="delete", url=url)

        return data

    """
    Private Functions to support Public Functions
    """

    def _supported_app_metadata(self, **kwargs):
        """
        Validate Metadata
        """
        data = {}
        if "categories" in kwargs:
            categories = ["Monitoring", "Security", "IOT", "Others"]
            if kwargs["categories"] in categories:
                data["categories"] = []
                data["categories"].append(kwargs["categories"])
                return True, data
            return False, None
        return False, None

    def _request(self, **kwargs):
        """
        HTTP Requests
        """

        if "auth" in kwargs["type"].lower():
            url = kwargs["url"]
            headers = {
                "Content-Type": "application/json",
                "Accept": "application/json",
            }
            response = requests.request(
                "POST",
                url,
                auth=HTTPBasicAuth(
                    self.settings["dnac"]["username"],
                    self.settings["dnac"]["password"],
                ),
                headers=headers,
                verify=self.settings["dnac"]["secure"],
            )
            if response.ok:
                data = response.json()
            else:
                raise Exception(f"Can't login to Cisco DNA Center ({response.json()})")
            return data

        if "get" in kwargs["type"].lower():
            url = kwargs["url"]
            headers = {
                "X-Auth-Token": self.settings["dnac"]["token"],
                "Content-Type": "application/json",
                "Accept": "application/json",
            }
            response = requests.request(
                "GET",
                url,
                headers=headers,
                verify=self.settings["dnac"]["secure"],
            )
            if response.ok:
                data = response.json()
            else:
                if response.status_code == 404:
                    raise Exception(
                        f"Error: Image not found in Cisco DNA Center App Hosting Repository"
                    )
                raise Exception(f"Error: ({response.content})")
            return data

        if "post" in kwargs["type"].lower():
            url = kwargs["url"]
            tar = kwargs["tar"]
            headers = {
                "X-Auth-Token": self.settings["dnac"]["token"],
                "Content-Type": "multi-part/form-data",
                "Accept": "application/json",
            }
            multi_part = MultipartEncoder(
                fields={
                    "filename": tar,
                    "file": (tar, open(tar, "rb"), "application/x-tar"),
                }
            )
            response = requests.request(
                "POST",
                url,
                data=multi_part,
                headers={
                    "Content-Type": multi_part.content_type,
                    "X-Auth-Token": self.settings["dnac"]["token"],
                },
                verify=self.settings["dnac"]["secure"],
            )
            if response.ok:
                data = response.json()
            else:
                data = xmltodict.parse(response.content)
                raise Exception(
                    f"Error ({data['error']['code']}): {data['error']['description']}"
                )
            return data

        if "put" in kwargs["type"].lower():
            url = kwargs["url"]
            payload = json.dumps(kwargs["payload"])
            headers = {
                "X-Auth-Token": self.settings["dnac"]["token"],
                "Content-Type": "application/json",
                "Accept": "application/json",
            }

            response = requests.request(
                "PUT",
                url,
                headers=headers,
                data=payload,
                verify=self.settings["dnac"]["secure"],
            )
            if response.ok:
                data = response.json()
            else:
                raise Exception(f"Error: Problem to update app - {response.content}")
            return data

        if "delete" in kwargs["type"].lower():
            url = kwargs["url"]
            headers = {
                "X-Auth-Token": self.settings["dnac"]["token"],
                "Content-Type": "application/json",
            }
            response = requests.request(
                "DELETE",
                url,
                headers=headers,
                verify=self.settings["dnac"]["secure"],
            )
            if response.ok:
                return True
            else:
                return False

        return



import os
import json
import click
import yaml
import ciscodnacapphosting


"""ciscodnacapphosting  Console Script.
Copyright (c) 2020 Cisco and/or its affiliates.
This software is licensed to you under the terms of the Cisco Sample
Code License, Version 1.1 (the "License"). You may obtain a copy of the
License at
               https://developer.cisco.com/docs/licenses
All use of the material herein must be in accordance with the terms of
the License. All rights not expressly granted by the License are
reserved. Unless required by applicable law or agreed to separately in
writing, software distributed under the License is distributed on an "AS
IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
or implied.
"""

""" Main Entry """


@click.group(chain=True)
@click.version_option()
@click.pass_context
def cli(ctx):
    pass


""" Display current config """


@cli.command("whoami")
@click.pass_context
def whoami(ctx):
    if "DNAC_CONFIG" in os.environ:
        config = ciscodnacapphosting.Api.config(
            config=os.environ["DNAC_CONFIG"], operation="decode"
        )
        click.echo(f"Config: {json.dumps(config[1])}")
        return
    else:
        config = ciscodnacapphosting.Api.config(operation="read")
        click.echo(f"Config: {config[1]}")
        return


""" Configures Cisco DNA Center settings """


@cli.command("config")
@click.option("--hostname", required=True)
@click.option("--username", required=True)
@click.option("--password", required=True)
@click.option("--secure/--insecure", default=True)
@click.option("--encode/--no-decode", default=False)
@click.pass_context
def dnac_config(ctx, hostname, username, password, secure, encode):
    if encode is True:
        config = ciscodnacapphosting.Api.config(
            hostname, username, password, secure, operation="encode"
        )
    else:
        config = ciscodnacapphosting.Api.config(
            hostname, username, password, secure, operation="write"
        )
    if config[0] is True:
        click.echo("Success: Config Updated")
        if encode is True:
            click.echo(f"Config Encode: {config[1]}")
    else:
        click.echo("Error: Config couldn't be updated")
    return


""" Get Application(s) from Cisco DNA Center """


@cli.command("app")
@click.option("--id", required=False)
@click.option("--image", required=False)
@click.option("--tag", required=False)
@click.pass_context
def app(ctx, id, image, tag):
    dnac_app = ciscodnacapphosting.Api()
    if id != None:
        click.echo(f"Get App ({id})")
        if tag != None:
            app = dnac_app.get(appId=id, tag=tag)
            click.echo(
                yaml.safe_dump(app, allow_unicode=True, default_flow_style=False)
            )
            return
        app = dnac_app.get(appId=id)
        click.echo(yaml.safe_dump(app, allow_unicode=True, default_flow_style=False))
        return
    if image != None:
        click.echo(f"Get App ({image})")
        if tag != None:
            app = dnac_app.get(image=image, tag=tag)
            click.echo(
                yaml.safe_dump(app, allow_unicode=True, default_flow_style=False)
            )
            return
        app = dnac_app.get(image=image)
        click.echo(yaml.safe_dump(app, allow_unicode=True, default_flow_style=False))
        return
    click.echo(f"Get App list")
    apps = dnac_app.get()
    for app in apps["data"]:
        click.echo(yaml.safe_dump(app, allow_unicode=True, default_flow_style=False))
    return


""" Upload Applications to Cisco DNA Center """


@cli.command("upload")
@click.option("--file", required=True)
@click.option("--categories", required=True)
@click.pass_context
def upload(ctx, file, categories):
    dnac_app = ciscodnacapphosting.Api()
    click.echo(f"Upload App ({file}) - {categories}")
    upload = dnac_app.upload(tar=file, categories=categories)
    click.echo(
        f"New AppId ({upload['appId']}) {upload['name']}:{upload['version']} - {categories}"
    )
    return


""" Upgrade Applications to Cisco DNA Center """


@cli.command("upgrade")
@click.option("--id", required=True)
@click.option("--tag", required=False)
@click.option("--file", required=True)
@click.option("--categories", required=True)
@click.pass_context
def upgrade(ctx, id, tag, file, categories):
    dnac_app = ciscodnacapphosting.Api()
    if tag != None:
        click.echo(f"Upgrade App ({file}) - {tag} - {categories}")
        upgrade = dnac_app.upgrade(appId=id, tag=tag, tar=file, categories=categories)
    else:
        click.echo(f"Upgrade App ({file}) - latest - {categories}")
        upgrade = dnac_app.upgrade(appId=id, tar=file, categories=categories)
    click.echo(
        f"New AppId {upgrade['appId']} of {upgrade['name']}:{upgrade['version']}"
    )
    return


""" Update Metadata about Applications to Cisco DNA Center """


@cli.command("update")
@click.option("--id", required=True)
@click.option("--categories", required=True)
@click.pass_context
def update(ctx, id, categories):
    dnac_app = ciscodnacapphosting.Api()
    click.echo(f"Update App ({id}) - {categories}")
    update = dnac_app.update(appId=id, categories=categories)
    click.echo(f"Update App ({update['appId']}) {update['name']} - categories")
    return


""" Delete Application(s) from Cisco DNA Center """


@cli.command("delete")
@click.option("--id", required=True)
@click.option("--tag", required=False)
@click.pass_context
def delete(ctx, id, tag):
    dnac_app = ciscodnacapphosting.Api()
    if tag != None:
        click.echo(f"Delete App ({id}) - {tag}")
        delete = dnac_app.delete(appId=id, tag=tag)
        click.echo(f"Deleted App ({id}) - {tag}")
    else:
        click.echo(f"Delete App ({id})")
        delete = dnac_app.delete(appId=id)
        click.echo(f"Deleted App ({id})")
    return


""" Wrapped docker cli to download and save """


@cli.command("docker")
@click.option("--download", required=False)
@click.option("--save/--no-save", default=False)
def docker(download, save):
    dnac_app = ciscodnacapphosting.Api()
    if download == None and save is False:
        download = click.prompt("Docker image")
        save = True
        click.echo(f"Downloading {download} and saving it locally")
    if download != None:
        if ":" in download:
            download = download.split(":")
            docker_download = dnac_app.docker.download(
                image=download[0], tag=download[1]
            )
        else:
            docker_download = dnac_app.docker.download(image=download, tag="latest")
    if save is True:
        save = dnac_app.docker.save(
            image=docker_download["image"], tag=docker_download["tag"]
        )
        click.echo(
            f"Download completed ({save['image']}) - saved as {save['filename']}"
        )
    return


if __name__ == "__main__":
    cli()
